; =================
; Low level link include
; =================

; Available routines:

;    = Writing
;        - setHighHigh
;        - setHighLow
;        - setLowHigh
;        - setLowLow
;        - setTipLow
;        - setTipHigh
;        - setRingLow
;        - setRingHigh
;        - toggleTip

;    = Reading
;        - readLines
;        - readTip
;        - readRing

; ==============================
; Defines for different calculator models
; ==============================

.if platform == ti8x
	.define TI83P
.elseif platform == ti83
	.define TI83
.endif

.ifdef TI83P
	.define readtipbit 0
	.define readringbit 1
   
	.define writebasevalue 0
	.define writetipbit 0
	.define writeringbit 1
.endif

.ifdef TI83
	.define readtipbit 2
	.define readringbit 3
   
	.define writebasevalue $D0
	.define writetipbit 0
	.define writeringbit 1
.endif

; ==============
; Writing to the port
; ==============

; Set both lines high (+5 volt, default setting)

setHighHigh:
	
	ld a,writebasevalue
	out (0),a
	ret

; Pull both lines low (0 volt)

setLowLow:

	ld a,writebasevalue|(1<<writetipbit)|(1<<writeringbit)
	out (0),a
	ret

; Pull the tip low, leave the ring high

setLowHigh:
	
	ld a,writebasevalue|(1<<writetipbit)
	out (0),a
	ret
	
; Pull the ring low, leave the tip high

setHighLow:

	ld a,writebasevalue|(1<<writeringbit)
	out (0),a
	ret
	
; Set tip low without changing the ring

setTipLow:

	call readRing
	jp z,setLowLow
	jp setLowHigh
	
; Set tip high without changing the ring

setTipHigh:

	call readRing
	jp z,setHighLow
	jp setHighHigh
	
; Set ring low without changing the tip

setRingLow:

	call readTip
	jp z,setLowLow
	jp setHighLow

; Set ring high without changing the tip

setRingHigh:

	call readTip
	jp z,setLowHigh
	jp setHighHigh

; Toggle tip high/low without changing the ring

toggleTip:

	call readTip
	jp z,toggleTip_islow
	jp setTipLow
toggleTip_islow:
	jp setTipHigh
	
; =================
; Reading from the port
; =================

; Read the state of the lines, model independant
; Returns value in a representing the state;
;    0   Both lines are high
;    1   Tip is low, ring is high
;    2   Ring is low, tip is high
;    3   Both lines are low
	
readLines:

	ld b,0
	in a,(0)
	bit readtipbit,a
	call z,readLines_tiplow
	bit readringbit,a
	call z,readLines_ringlow
	ld a,b
	ret
	
readLines_tiplow:
	ld b,1
	ret
readLines_ringlow:
	push af
	ld a,b
	or 2
	ld b,a
	pop af
	ret

; Read the state of the tip
; Returns state in z flag;
;     z    Tip is low
;     nz  Tip is high

readTip:
	
	in a,(0)
	bit readtipbit,a
	ret
	
; Read the state of the ring
; Returns state in z flag;
;     z    Ring is low
;     nz  Ring is high

readRing:
	
	in a,(0)
	bit readringbit,a
	ret

lowlevelend:

.echo "Low level link included, size is "
.echo lowlevelend-setHighHigh
.echo " bytes\n"
